{
This module is the Assembly language (ADSP21xx family processors) routine to
perform ADC-DAC looping.
We assume that the system uses 12 bit ADC (AD574A) and 12 bit DAC. For AD
conversion we need to give Start of the Conversion pulse to the ADC, this is
achieved by performing a dummy write to the ADC.
The ADC, Start of Conversion, DAC are all mapped in the memory space of the
processor as given in the system file for the DSP programs in this book.
The system file name is Example.sys.
}

.MODULE     AD2DALP              {Name of the module for ADC to DAC Looping}

{
The constants such as address of A2D conversion ADC read, DAC write, Timer
register which are constant throughout the program. 
}
.CONST      Start_Of_Conv = 0x0002;
            {Address for giving strobe to a/d converter}
.CONST      ADC_ADDR =      0x0001;
            {The ADC data is read from this address}
.CONST      DAC_ADDR =     0x0003;
            {Address for writing the data to the DAC}
.CONST      tperiod =       0x3ffd;
            {Address of Timer period register, Value at this location is 
            loaded in the tcount when the timer expires}
.CONST      tcount =        0x3ffc;
            {Address of Timer counter, in which the count is loaded}
.CONST      tscale =        0x3ffb;
            {Address of Scaling register, is used to decide how many
            times the count will repeat}
{****************************************************************************}
{
After declaration of the constants, the Interrupt vectro table is written.
On reset the processor boots from the boot EPROM and starts executing from
the 0x0000 location where interrupt vector table resides.
}
start:  JUMP Begin;     RTI;    RTI;    RTI;  {on reset jump to Begin}
        RTI;            RTI;    RTI;    RTI;  {sport1 transmit interrupt LOC}
        RTI;            RTI;    RTI;    RTI;  {sport1 reciever interrupt LOC}
        JUMP Tmr_ISR;   RTI;    RTI;    RTI;  {On timer interrupt goto Tmr_ISR}

{****************************************************************************}
{
Main routine of the module in which all the Initialisations are done.
}

Begin:
        AX0 = 0x3E8;
        {This count will give a sampling frequency of 10 kHz}

        DM(tperiod)=AX0;        {load Tperiod and tcount with same count}
        DM(tcount)=AX0;         
		AX0 = 0;
        DM(tscale)=AX0;         {set scaling register = 0 No scaling}
{
When you write 1 to a bit in ICNTL reg. you make that interrupt edge
sensetive. Similarly when you set a bit in the IMASK reg. you enable
that particular interrupt.
}
        ICNTL = b#10111;            {All interrupts edge sensitive}

        {Enable Timer intterupt and disable all  other interrupts}

        IMASK = b#100001;
        AX1 = 0x0000;
        DM(Start_Of_Conv) = AX1;    {Give start of Convertion pulse to ADC}
        ENA TIMER;                  {enable the timer }
Wait_here:   JUMP Wait_here;        {Wait here forever}

{
The program comes out of this loop only when the timer count expires and
the timer causes and intterupt.
}

{****************************************************************************}
{
Timer interrupt service routine. In this routine the processor reads data from
the ADC, masks the upper 4 bits and writes the data back to the DAC.
At the same time  it gives a start of conversion pulse to the ADC, this done
to get next dat on next timer interrupt.
}
Tmr_ISR: AY0 = 0X0FFF;
        {We are using 12 bit ADC's hence AY0 is used for masking upper 4 bits}

        AX0 = DM(ADC_ADDR);     {Read the ADC data}

        {Give again Start of Conversion pulse to get next data}
        DM(Start_Of_Conv) = AX1;
        
        {Mask upper 4 bits to get 12 bit data}
		AR = AX0 AND AY0;
        
        {Write the data to the DAC}
        DM(DAC_ADDR) = AR;     
        RTI;                {Return from the interrupt service routine.}

.endmod;

{End of the AD2DALP module}
