/*
About this program:
This program is used to design an IIR filter by using bilinear
transformation method. 
In this program the filter designed is 2nd oder filter.
In biliner transformation the transfer function of the filter in
S domain (Laplace variable S) is given. The corrosponding transfer
function in Z domain is obtained by replacing S by
            2 (1 - Z^-1)
            ------------
            T (1 + Z^1 )
2/T is a factor depending on the resonant fequency of the filter.
You have to input the values of a and b in the following equation
                       s + a
         H(s) =  ------------------ 
                  (s + a)^2  +  b

from which the factor 2/T is derived.

*/
#include<stdio.h>
#include<conio.h>
#include<stdlib.h>
#include<ctype.h>
#include<graphics.h>
#include<math.h>

#define OriginX  320
#define OriginY  240
#define Radius   200
#define Pole 1
#define Zero 0

float ZeroY,ZeroX;
float r1,i1,r2,i2;
int gd,gm;
/*
To find roots of a quadratic equation.
*/
int quadratic(float,float);
/*
To draw pole zeros on the Z plane.
*/
void DrawPolezero(int);

main()
{
  int i,j,retval,Xnum,Ynum;
  int flag;
  float a,b,wr,W,Tau;
  float SmagNum[3],SmagDenom[3],ZmagNum[3],ZmagDenom[3];
  float ZnReal[3],PnReal[3],ZnImg[3],PnImg[3],Temp,Temp1;
  FILE *output;
  /*
  Set all the arrays used in the program to zero.
  */
  for(i = 0;i < 3;i++)
  {
        ZnReal[i] = 0;
        ZnImg[i] = 0;
        PnReal[i] = 0;
        PnImg[i] = 0;

        SmagNum[i] = 0;
        SmagDenom[i] = 0;
        ZmagNum[i] = 0;
        ZmagDenom[i] = 0;
  }
  clrscr();
  for(j=0;j<80;j++)
        printf("");
  printf("\t\tBilinear Transformation of 2nd order filter.\n");
  for(j=0;j<80;j++)
        printf("");

  printf("\nThe system function of a 2nd order filter in s domain is given as,"
	 "\n              s + a"
	 "\nH(s) =  ------------------ "
	 "\n         (s + a)^2  +  b   "
	 "\nWhere, a = +ve or -ve"
	 "\n       b = +ve value."
	 "\nThe desired filter is obtained by replacing"
	 "\n         2 (1 - Z^-1)"
	 "\n  s by   ------------"
	 "\n         T (1 + Z^1 )");

  printf("\nInput the value of a: ");
  scanf("%f",&a);
  printf("\nInput the value of b: ");
  scanf("%f",&b);
  printf("\nInput the value of resonant frequency Wr(rad.): ");
  scanf("%f",&wr);

  W = sqrt(b);
  Tau = W/tan(wr/2);
  b = Tau;
  clrscr();
  for(j=0;j<80;j++)
        printf("");
  printf("\t\tBilinear Transformation of 2nd order filter.\n");
  for(j=0;j<80;j++)
        printf("");
  printf("\nWe get the value of mapping factor (2/T) = %f",Tau);

  /*
  Calculate the filter coefficients in S domain.
  */
  /*
  Numerator terms.
  */
  SmagNum[2] = 0;
  SmagNum[1] = 1;
  SmagNum[0] = a;
  /*
  Denomenator terms.
  */
  SmagDenom[2] = 1.0;
  SmagDenom[1] = 2 * a;
  SmagDenom[0] = a * a + b * b;
  /*
  Calcuate the coefficients of Z terms.
  Numerator Terms.
  */
  ZmagNum[2] = Tau + a;
  ZmagNum[1] = 2 * a;
  ZmagNum[0] = a - Tau;
  /*
  Denomeanator terms.
  */
  ZmagDenom[2] = a * a + 2 * a * Tau +  b * b;
  ZmagDenom[1] = 2 * a * a + 2 * b * b - 2 * Tau;
  ZmagDenom[0] = 1 - 2 * a * Tau + a * a + b * b;

  /*
  Calculate the filter coefficients in Z domain and represent
  them as difference equation coefficients.
  */
  Ynum = 2;
  Xnum = 3;
  Temp = ZmagDenom[2];
  ZmagDenom[2] = ZmagDenom[2] / Temp;
  ZmagDenom[1] = ZmagDenom[1] / Temp;
  ZmagDenom[0] = ZmagDenom[0] / Temp;

  ZmagNum[2] = ZmagNum[2] / Temp;
  ZmagNum[1] = ZmagNum[1] / Temp;
  ZmagNum[0] = ZmagNum[0] / Temp;

  Temp = ZmagDenom[1];
  Temp1 = ZmagDenom[0];
  /*
  Find the roots of the quaratic equation to get the location of the poles
  of the system.
  */
  retval = quadratic(Temp, Temp1);
  PnReal[0] = r1;
  PnImg[0] = i1;
  PnReal[1] = r2;
  PnImg[1] = i2;

  Temp = ZmagNum[1]/ZmagNum[2];
  Temp1 = ZmagNum[0]/ZmagNum[2];

  /*
  Find the roots of the quaratic equation to get the location of the zeros
  of the system.
  */
  retval = quadratic(Temp, Temp1);
  ZnReal[0] = r1;
  ZnImg[0] = i1;
  ZnReal[1] = r2;
  ZnImg[1] = i2;

  /*
  Write the coefficients of the difference equation for the filter
  to the file.
  */
  output = fopen("coeff.dat","w");
  if(output == NULL)
  {
	printf("\n Can't Open File Coeff.dat");
	exit(0);
  }
  fprintf(output,"%d",Ynum + 1);
  fprintf(output,"\n%d",Ynum);
  clrscr();

  printf("\nThe Filter coefficients are:"
	 "\nX coefficients: ");
  for(i = 0;i <= Ynum; i++)
  {
        /*
        Write in the order coefficients of x(n), x(n-1) and so on.
        where x(n) specifies current input.
        */
        fprintf(output,"\n%f",ZmagNum[Ynum - i]);
        printf("\n%f",ZmagNum[Ynum -i]);
  }
  printf("\nY coefficients: ");
  for(i = 1;i <= Ynum; i++)
  {
        /*
        Write in the order coefficients of y(n-1), y(n-2) and so on.
        where y(n) is current output.
        */

        fprintf(output,"\n%f",ZmagDenom[Ynum - i]);
        printf("\n%f",ZmagDenom[Ynum - i]);
  }
  /*
  Close the file.
  */
  fclose(output);

  getch();
/*---------------------------------------------------------------------*/
/*
The code after this is used to plot the poles and zeros  in the z plane.
It's presentation part of the program and readers needn't pay any attention
to it. For explaination reders can refer to the program coef2plt.c
*/

	 detectgraph(&gd,&gm);
	 initgraph(&gd,&gm,"");
	 graphresult();
         cleardevice();
         setbkcolor(0);
	 circle(OriginX,OriginY,Radius);
	 line(OriginX - 300,OriginY,OriginX + 300, OriginY);
	 line(OriginX,OriginY - 230, OriginX, OriginY + 230);
	 outtextxy(OriginX - 100,OriginY - 225,"Imaginary(I)");
	 outtextxy(OriginX + 220 ,OriginY + 5,"Real(X)");
	 outtextxy(OriginX + 15, OriginY + 10, "Z = 0");
         outtextxy(OriginX - 230, OriginY + 150,"Unit Circle");
	 settextstyle(0,0,2);
	 outtextxy(OriginX - 300, OriginY + 210,"Pole Zero Plot");
	 for(i = 0;i < Xnum - 1; i++)
         {
		ZeroX = ZnReal[i];
		ZeroY = ZnImg[i];
		flag = Zero;
		DrawPolezero(flag);
         }
         for(i = 0;i < Ynum; i++)
         {
		ZeroX = PnReal[i];
		ZeroY = PnImg[i];
		flag = Pole;
		DrawPolezero(flag);
         }
         setcolor(15);
         getch();
	 closegraph();
}
/*
This function is used find roots of the quadratic equation.
The function is same as the one used in the program coef2plt.c
Readers can refer to that program for explaination of the function.
*/
int quadratic(float r, float s)
{
  int  err;
  float decider;

  err = 0;

  decider = r * r + 4 * s;
  if(decider > 0)
  {
	r1 = (r + sqrt(decider))/2.0;
	r2 = (r - sqrt(decider))/2.0;
	i1 = 0;
	i2 = 0;
  }
  else
  {
	r1 = r/2;
	r2 = r1;
	decider = -1 * decider;
	i1 = sqrt(decider)/2.0;
	i2 = -i1;
  }

  return err;
}

/*
This function is used to draw poles and zeros in the Z plane,
accoring to the real and imaginary value of the pole or zero.
For explaination of this function refer to coef2plt.c
*/
void DrawPolezero(int flag)
{
 float Tempval;
 int Xpos,Ypos;
	 Tempval = 200.00 * ZeroX;
	 Xpos = (int) Tempval;
	 Tempval = 200.00 * ZeroY;
	 Ypos = (int) Tempval;
	 setcolor(3);
	 if(ZeroX == 0 || ZeroY == 0)
		setlinestyle(0,0,2);
	 else
		setlinestyle(0,0,0);
         if(flag == Zero)
         {

		circle(OriginX + Xpos, OriginY + Ypos,4);
                circle(OriginX + Xpos, OriginY - Ypos,4);
         }
         else
         {
                line(OriginX + Xpos - 4,OriginY + Ypos - 5,OriginX + Xpos + 4,OriginY + Ypos + 5);
                line(OriginX + Xpos - 4,OriginY + Ypos + 5,OriginX + Xpos + 4,OriginY + Ypos - 5);
                line(OriginX + Xpos - 4,OriginY - Ypos - 5,OriginX + Xpos + 4,OriginY - Ypos + 5);
                line(OriginX + Xpos - 4,OriginY - Ypos + 5,OriginX + Xpos + 4,OriginY - Ypos - 5);

         }
         line(OriginX,OriginY,OriginX + Xpos, OriginY - Ypos);
         line(OriginX,OriginY,OriginX + Xpos, OriginY + Ypos);
}