/*
About this program:
This program is used to calculate the poles and zeros of the system
from the coefficients of the difference equation of the system are given.
The difference equation is given as,
y(n) = a0 * x(n) + a1 * x(n-1) + a2 * x(n-2) + .... + aP * x(n-P)
        + b1 * y(n-1) + b2 * y(n-2) + .... + bQ * y(n-Q)
where y(n) and x(n) represent current output and input respectively.

The user has to input the values P and Q-1 (b0 doesn't exist) and the
corrosonding a and b coefficient values.
The program can also read above values from the default coefficient file
coeff.dat.

After reading the coefficients the program generates the polynomials of Z
from the system function given as,

        a0 + a1 * Z^-1 + a2 * Z^-2 + a3 * Z^-3 + .... + aP * Z^-P
H(Z) = --------------------------------------------------------------
         1 + b1 * Z^-1 + b2 * Z^-2 + b3 * Z^-3 + .... + bQ * Z^-Q

These polynomials in Z are solved using LinBairstow method of obtaining
roots of the polynomial to get the position of poles and zeros in terms
of real and imaginary components.

The Advantage of LinBairstow method is that it can calculate roots of
the polynomial as both real and imaginary numbers.

The poles and zero's thus obtained can be plotted on the Z plane.
*/
#include<stdio.h>
#include<conio.h>
#include<graphics.h>
#include<math.h>
#include<stdlib.h>
#define OriginX  320
#define OriginY  240
#define Radius   200
#define Pole 1
#define Zero 0

int gd,gm;
float ZeroY,ZeroX;
float a[20],re[20],im[20];
float r1,i1,r2,i2;
float Xcoeff[20],Ycoeff[20];
int Xnum,Ynum;
/*
Reading the coefficients from the file.
*/
int ReadCoefFile();
/*
Reading the coefficients from the Console.
*/
int ReadCoef();

/*
To obtain roots of polynomials.
*/
int bairstow(int,float,float,float,int);

/*
To obtain roots of quaratic equations.
*/
int quadratic(float,float);

/*
To draw poles and zeros on the unit circle.
*/
void DrawPolezero(int);
main()
{
	int i,j,k,nn,maxit,retval,TempNum,XrootNum,YrootNum;
        int Choice,flag;
	float es,rr,ss,Xreal[20],Ximg[20],Yreal[20],Yimg[20];
	float Amplitude,phase;
        clrscr();
        /*Leave----------------------------*/
        for(j=0;j<80;j++)
                printf("");
	printf("\t\t\t\tPole Zero Plot.\n");
        for(j=0;j<80;j++)
                printf("");
        /*---------------------------------*/

	fflush(stdin);
	printf("\nWhat is the format of Input?"
	       "\n1. File."
	       "\n2. Console"
	       "\nInput Your Choice: ");
	scanf("%d",&Choice);
	fflush(stdin);
        /*
        Select whether to read the coefficients from the file or
        the console.
        */
	switch(Choice)
	{
	  case 1:
		fflush(stdin);
                ReadCoefFile();
		break;
	  case 2:
                ReadCoef();
		break;
	  Default:
		printf("Invalid Choice !! \nPress any key to Continue.");
		getch();
                ReadCoef();
	}
        /*
        We have the a coefficients in the Xcoeff array, the power of the
        equation will be P (number of X coeff Xnum - 1), as shown in H(Z)
        above.
        a0 will be coefficient of Z^P hence will have to reverse the array
        Xcoeff[i] while sending it to the function bairstow.
        */

        maxit = 1000;                    /* Maximum iterations allowed=1000*/
        /*
        Set the arrays to zero before calling the function.
        */

	for(i = 0; i< 20; i++)
	{
		re[i] = 0;
		im[i] = 0;
		a[i] =  0;
	}
        /*
        Power of the polynomial.
        */
	nn = Xnum - 1;
        /*
        Reverse the array.
        */
	for(i = 1;i <= Xnum;i++)
	{
	  a[Xnum - i] = Xcoeff[i - 1];
	}
        /*
        Set error margin in root calculations to 0.1%
        */
	es = 0.1;
	rr = 0;
	ss = 0;
        /*
        Decrese the power of polynomial if the coefficients of lower powers of
        z are zero. This incraeses the accuracy.
        */

	TempNum = nn;
	while(a[0] == 0)
	{
                TempNum--;
                for(j=0; j <= nn; j++)
                a[j] = a[j+1];
	}
	nn = TempNum;
        /*
        Call the bairstow function to calculate roots of the
        polynomial.
        */
	retval = bairstow(nn,es,rr,ss,maxit);
        /*
        Real and imaginary roots are obtained in the arrays re and
        im respectively store them in the result arrays.
        */
	XrootNum = nn;
	for(i = 0; i<= nn; i++)
	{
		Xreal[i] = re[i];
		Ximg[i] = im[i];
	}

        /*
        Repeat the above procedure for b coefficients.
        */
	for(i = 0; i< 20; i++)
	{
		a[i] = 0;
                re[i] = 0;
                im[i] = 0;
	}
	nn = Ynum;
	for(i = 0;i<= Ynum;i++)
	{
		a[Ynum - i] =  Ycoeff[i];
	}
	maxit = 1000;
        /*
        error margin = 1%
        */
	es = 1.0;
	rr = 0;
	ss = 0;
        /*
        Call bairstow function only if the system has any poles at locations
        other than origin in Z plane.
        */
	if (nn > 0)
		retval = bairstow(nn,es,rr,ss,maxit);

        /*
        store roots in the result arrays.
        */
        YrootNum = nn;
	for(i = 0; i<= nn; i++)
	{
		Yreal[i] = re[i];
		Yimg[i] = im[i];
	}
        /*Leave-------------------------------*/
	clrscr();
        for(j=0;j<80;j++)
                printf("");
	printf("\t\t\t\tPole Zero Plot.\n");
        for(j=0;j<80;j++)
		printf("");
        /*------------------------------------*/

        /*
        Print the result on the screen in the format,
        Real component, Imaginary component, Magnitude and Phase angle.
        */
	printf("\nThe Zero's of the system are: ");
	printf("\n\tReal\t\tImaginary\t\tR\t\t(rad)");

	for(i= 1;i <= XrootNum; i++)
	{
                Amplitude = sqrt(Xreal[i] * Xreal[i] + Ximg[i] * Ximg[i]);
		if(Xreal[i] != 0)
                        phase =  atan(Ximg[i]/Xreal[i]);
                else
			if(Ximg[i] > 0)
				phase = 22.0/14.0;
			else
				phase = -22.0/14.0;
		printf("\nZ%d = %0.2f\t\t%0.2f",i,Xreal[i],Ximg[i]);
		printf("\t\t\t%0.2f\t\t%0.2f",Amplitude,phase);
	}
	getch();

	if(nn > 0)
	{
		clrscr();
		for(j=0;j<80;j++)
			printf("");
		printf("\t\t\t\tPole Zero Plot.\n");
		for(j=0;j<80;j++)
			printf("");
		printf("\nThe Pole's of the system are: ");
		printf("\n\tReal\t\tImaginary\t\tR\t\t(rad)");

		for(i= 1;i <= YrootNum; i++)
		{
			Amplitude = sqrt(Yreal[i] * Yreal[i] + Yimg[i] * Yimg[i]);
			if(Yreal[i] != 0)
				phase =  atan(Yimg[i]/Yreal[i]);
			else
				if(Yimg[i] > 0)
					phase = 22.0/14.0;
				else
					phase = -22.0/14.0;
			printf("\nZ%d = %0.2f\t\t%0.2f",i,Yreal[i],Yimg[i]);
			printf("\t\t\t%0.2f\t\t%0.2f",Amplitude,phase);
		}
		getch();
        }
/*
The code after this is used to plot the poles and zeros  in the z plane.
It's presentation part of the program and readers needn't pay any attention
to it. 
*/

         /*
         Initailise the graphics mode.
         */
         detectgraph(&gd,&gm);
         initgraph(&gd,&gm,"");
         graphresult();
         cleardevice();
         setbkcolor(0);
         /*
         Draw unit circle. Magnified radius = 200.
         */
         circle(OriginX,OriginY,Radius);
         /*
         Draw real and imaginary axes.
         */
         line(OriginX - 300,OriginY,OriginX + 300, OriginY);
         line(OriginX,OriginY - 230, OriginX, OriginY + 230);
         outtextxy(OriginX - 100,OriginY - 225,"Imaginary(I)");
         outtextxy(OriginX + 220 ,OriginY + 5,"Real(X)");
         outtextxy(OriginX + 15, OriginY + 10, "Z = 0");
         outtextxy(OriginX - 230, OriginY + 150,"Unit Circle");
         settextstyle(0,0,2);
         outtextxy(OriginX - 300, OriginY + 210,"Pole Zero Plot");
         /*
         Call the procedure DrawPolezero to plot zeros. The position of
         zeros  is given in ZeroX and ZeroY. flag is used to indicate
         whether it is a pole or zero.
         */
	 for(i = 1;i <= XrootNum; i++)
         {
		ZeroX = Xreal[i];
		ZeroY = Ximg[i];
                flag = Zero;
                DrawPolezero(flag);
	 }

         /*
         Call the procedure DrawPolezero to plot poles. The position of
         poles  is given in ZeroX and ZeroY. flag is used to indicate
         whether it is a pole or zero.
         */
	for(i = 0;i <= YrootNum; i++)
	{
		ZeroX = Yreal[i];
		ZeroY = Yimg[i];
		flag = Pole;
		DrawPolezero(flag);
	}
	setcolor(15);
	getch();
	closegraph();

}
/*
This function is used to read the coefficients from the console.
The user has to input no. of a and b coefficients as well as their values.
The a coefficients are stored in the Xcoeff array.
The b coefficients are stored in the Ycoeff array.

*/
int ReadCoef()
{
  int i,j,checkflag;
  /*
  Read the number of a and b coefficients.
  */
  printf("\nInput the Number of X coefficients: ");
  scanf("%d",&Xnum);
  printf("\nInput the Number of Y coefficients: ");
  scanf("%d",&Ynum);
  /*
  No of poles in the system shouldn't be greater then number of zeros.
  */
  if(Xnum <= Ynum)
  {
	printf("\nNo. of X coefficients should be greater than No."
	       "\n of Y coefficients.");
	exit(0);
  }

  /*Leave--------------------------------*/
  clrscr();
  for(j=0;j<80;j++)
	printf("");
  printf("\t\t\t\tPole Zero Plot.\n");
  for(j=0;j<80;j++)
	printf("");
  /*-------------------------------------*/
  /*
  Read x coefficients.
  */
  printf("\nInput The X coefficients:\n");
  for(i = 0;i < Xnum;i++)
  {
     printf("Coefficient of Z^%d = ",-1 * i);
     scanf("%f",&Xcoeff[i]);
  }

  /*
  Read Y coefficients.
  */
  Ycoeff[0] = 1;
  checkflag = 0;
  printf("\nInput The Y coefficients:\n");
  for(i = 1;i <= Ynum;i++)
  {
     printf("Coefficient of Z^%d = ",-1 * i);
     scanf("%f",&Ycoeff[i]);
     if(Ycoeff[i] != 0)
	checkflag = 1;
  }
  /*
  If all b coefficients are zero, i.e. all poles are at origin make b coefficient
  number forcefully zero.
  */
  if(checkflag == 0)
	Ynum = 0;
  /*
  Return from the procedure.
  */
  return 0;
}

/*
This function is used to read the coefficients from the file coeff.dat
The a coefficients are stored in the Xcoeff array.
The b coefficients are stored in the Ycoeff array.
*/
int ReadCoefFile()
{
  int i,checkflag;
  FILE *input;
  input = fopen("coeff.dat","r");
  if(input == NULL)
  {
        /*
        Exit if file does not exist.
        */
        printf("Can't open the file coeff.dat");
	exit(0);
  }
  if(feof(input))
  {
	printf("Invalid file format");
	exit(0);
  }
  /*
  Read a and b coefficient number.
  */
  fscanf(input,"%d",&Xnum);
  if(feof(input))
  {
	printf("Invalid file format");
	exit(0);
  }
  fscanf(input,"%d",&Ynum);
  if(Xnum <= Ynum)
  {
	printf("\nNo. of X coefficients should be greater than No."
	       "\n of Y coefficients.");
	exit(0);
  }
  /*
  Read a coefficients.
  */
  for(i = 0;i < Xnum;i++)
  {
	if(feof(input))
	{
		printf("Invalid file format.");
		exit(0);
	}
	fscanf(input,"%f",&Xcoeff[i]);
  }
  /*
  Read b coefficients.
  */
  checkflag = 0;
  Ycoeff[0] = 1;
  for(i = 1;i <= Ynum;i++)
  {
        if(feof(input))
	{
		printf("Invalid file format.");
		exit(0);
	}
	fscanf(input,"%f",&Ycoeff[i]);
	if(Ycoeff[i] != 0)
		checkflag = 1;
  }
  if(checkflag == 0)
	Ynum = 0;
  return 0;
}
/*
This function is used to calculate the roots of the polynomial.
The polynomial is stored in array a while the roots are obtained in the
form of real and imaginary terms in arrays re and im respectively.
*/
int bairstow(int n,float es,float r,float s,int maxit)
{
  int ier,iter,i,retval;
  float ea1,ea2,b[10],c[10],det,temp,dr,ds;

  ea1 = 1.0;
  ea2 = 1.0;
  ier = 0;
  /*
  Solve the polynomial only if power is greater than 2.
  */
  if(n > 2)
  {
          /*
          set the iterations.
          */
          iter = 0;
	  do
          {
                iter = 0;
                do
                {
                        iter = iter + 1;
                        /*
                        Calculate the arrays b and c.
                        r snd s give initial predictions.
                        */
                        b[n] = a[n];
                        b[n - 1] = a[n - 1] + r * b[n];
			c[n] = b[n];
                        c[n - 1] = b[n - 1] + r * c[n];
                        /*
                        reduce the equation in array a to get one quaratic
                        out of it.
                        */
			for(i= n - 2;i >= 0; i--)
                        {
                                b[i] = a[i] + r * b[i + 1] + s * b[i + 2];
                                c[i] = b[i] + r * c[i + 1] + s * c[i + 2];
                        }
                        /*
                        solve the simultaneous equation to get error dr and ds.
                                c[2]dr + c[3]ds = -b[1]
                                c[1]dr + c[2]ds = -b[0]
                        */
                        det = c[2] * c[2] - c[3] * c[1];
                        if(det != 0)
                        {
				dr = (-1 * b[1] * c[2] + b[0] * c[3])/det;
				ds = (-1 * b[0] * c[2] + b[1] * c[1])/det;
                                r = r + dr;
                                s = s + ds;
                                if(r != 0)
                                {
                                        temp = dr/r;
                                        /*
                                        Get persent error.
                                        */
                                        ea1 = fabs(temp) * 100;
                                }
                                if(s != 0)
                                {
                                        temp = ds/s;
                                        /*
                                        Get persent error.
                                        */
                                        ea2 = fabs(temp) * 100;
                                }

                        }
                        else
                        {
                                r = r + 1;
                                s = s + 1;
                                iter = 0;
                        }
                        /*
                        Exit if maximum number of iterations is exceeded.
                        */
			if(iter > maxit)
				break;
                }while(ea1 > es && ea2 > es); /*End of while*/

                if(iter > maxit)
			break;
                /*
                Get the roots of the quaratic.
                */
                retval = quadratic(r,s);
                /*
                Store the roots.
                */
                re[n] = r1;
                im[n] = i1;
                re[n - 1] = r2;
                im[n - 1] = i2;
                /*
                reduced the power of the equation.
                the reduced equation is present in array b.
                */
                n = n - 2;

		for(i= 0;i <= n;i++)
                {
                        a[i] = b[i + 2];
		}

          }while(n > 2);    /*End of while*/
  }
  if(iter < maxit)
  {
        /*
        If power of the polynomail = 2. then call function quaratic to
        get its roots.
        */
        if(n == 2)
	{
		r = -a[1]/a[2];
		s = -a[0]/a[2];
		retval = quadratic(r,s);
                /*
                Store the roots.
                */
		re[n] =  r1;
		im[n] =  i1;
		re[n - 1] = r2;
		im[n - 1] = i2;
	}
        /*
        Power is one get the real root directly.
        */
	else
	{
		re[n] = -a[0]/a[1];
		im[n] = 0;
	}
  }
  else
	ier  = 1;
  /*
  return error status. 0 => OK 1 => ERROR.
  */
  return ier;
}

/*
This function is used find roots of the quadratic equation.
The roots are returened the variables r1, i1, and r2, i2.
*/
int quadratic(float r, float s)
{
  int  err;
  float decider;

  err = 0;
  /*
  Decide wherther roots are real or imaginary.
  */
  decider = r * r + 4 * s;
  if(decider > 0)
  {
        /*
        Calculate real roots and store in r1  and r2.
        */
        r1 = (r + sqrt(decider))/2.0;
	r2 = (r - sqrt(decider))/2.0;
	i1 = 0;
	i2 = 0;
  }
  else
  {
        /*
        Get the real and imaginary part of the roots.
        */
        r1 = r/2;
	r2 = r1;
	decider = -1 * decider;
	i1 = sqrt(decider)/2.0;
	i2 = -i1;
  }

  return err;
}
/*
This function is used to draw poles and zeros in the Z plane,
accoring to the real and imaginary value of the pole or zero,
which are given in variables ZeroX and ZeroY.
*/
void DrawPolezero(int flag)
{
 float Tempval;
 int Xpos,Ypos;
         /*
         Calulate value on real axis from ZeroX.
         */
         Tempval = 200.00 * ZeroX;
	 Xpos = (int) Tempval;

         /*
         Calulate value on Imaginary axis from ZeroY.
         */
	 Tempval = 200.00 * ZeroY;
	 Ypos = (int) Tempval;
	 setcolor(3);
	 if(ZeroX == 0 || ZeroY == 0)
                setlinestyle(0,0,4);
	 else
		setlinestyle(0,0,0);
         /*
         If the flag = 0 draw a circle to indicate zero
         at the point given by Xpos and Ypos.
         */
         if(flag == Zero)
         {

		circle(OriginX + Xpos, OriginY + Ypos,4);
                circle(OriginX + Xpos, OriginY - Ypos,4);
         }
         /*
         Otherwise draw a cross to indicate pole at position given by Xpos
         and Ypos.
         */
         else
         {
                line(OriginX + Xpos - 4,OriginY + Ypos - 5,OriginX + Xpos + 4,OriginY + Ypos + 5);
                line(OriginX + Xpos - 4,OriginY + Ypos + 5,OriginX + Xpos + 4,OriginY + Ypos - 5);
                line(OriginX + Xpos - 4,OriginY - Ypos - 5,OriginX + Xpos + 4,OriginY - Ypos + 5);
                line(OriginX + Xpos - 4,OriginY - Ypos + 5,OriginX + Xpos + 4,OriginY - Ypos - 5);

         }
         line(OriginX,OriginY,OriginX + Xpos, OriginY - Ypos);
         line(OriginX,OriginY,OriginX + Xpos, OriginY + Ypos);
         /*
         Return from the subroutine.
         */
}
