{
This module is the Assembly language (ADSP21xx family processors) routine to
perform difference equation implementation.
We assume that the system uses 12 bit ADC (AD574A) and 12 bit DAC. For AD
conversion we need to give Start of the Conversion pulse to the ADC, this is
achieved by performing a dummy write to the ADC.
The ADC, Start of Conversion, DAC are all mapped in the memory space of the
processor as given in the system file for the DSP programs in this book.
The system file name is Example.sys.

This module reads data for a coefficents and b coefficents from constant array
defined in the program.
After reading these coefficents it initialises necessary cirular buffers for
data input, data buffer  for past outputs and inputs.
The method of calculating the the output is multiply and add as explained in
the diffeq.c source code in this book.
The  total sum is  converted in to 12 bits for writting output on DAC.
}

.MODULE Diffeq;     {Name of the module for difference equation implementation}

{
The constants such as address of A2D conversion ADC read, DAC write, Timer
register which are constant throughout the program. 
}
.CONST      Start_Of_Conv = 0x0002;
            {Address for giving strobe to a/d converter}
.CONST      ADC_ADDR =      0x0001;
            {The ADC data is read from this address}
.CONST      DAC_ADDR =     0x0003;
            {Address for writing the data to the DAC}
.CONST      tperiod =       0x3ffd;
            {Address of Timer period register, Value at this location is 
            loaded in the tcount when the timer expires}
.CONST      tcount =        0x3ffc;
            {Address of Timer counter, in which the count is loaded}
.CONST      tscale =        0x3ffb;
            {Address of Scaling register, is used to decide how many
            times the count will repeat}
{****************************************************************************}
{
Declaration of variables required for the module.
}
.var/pm/ram/abs=0x1000          Bcoeff;
                    {Coefficients b which multiply with previous Y outputs
                     are stored here}
.var/pm/ram/abs=0x1100          Acoeff;
                    {Coefficients a which multiply with previous X inputs
                     and current X input are stored here}
                    

.var/dm/ram/abs=0x0200          Yout;
                {The previous Y outputs are stored here}
.var/dm/ram/abs=0x0300          Xinput;
                {The previous X inputs are stored here}

{****************************************************************************}
{
After declaration of the constants and the variables, the Interrupt vector
table is written.
On reset the processor boots from the boot EPROM and starts executing from
the 0x0000 location where interrupt vector table resides.
}
start:  JUMP Begin;     RTI;    RTI;    RTI;  {on reset jump to Begin}
        RTI;            RTI;    RTI;    RTI;  {sport1 transmit interrupt LOC}
        RTI;            RTI;    RTI;    RTI;  {sport1 reciever interrupt LOC}
        JUMP Tmr_ISR;   RTI;    RTI;    RTI;  {On timer interrupt goto Tmr_ISR}

{****************************************************************************}
{
Main routine of the module in which all the Initialisations are done.
Here we are performing the difference equation implementation with fixed
number of coefficients for X input and Y output.
the number of a coefficients for X inputs is 7 while number of b coefficients
for Y outputs is 6.
If we want to have lesser number of a or b coefficients we simply set rest
of the coefficients to zero.
}

Begin:    AX0 = 0x07;     {Number of X coefficients = 7}                         
          L6 = AX0;       {Circular buffer with length 7}
          M6 = 1;         {buffer stride  = 1}
          I6 = ^Acoeff;   {I6 points to the a coefficients}
        
          AX0 = 0x0000;        {1st a coefficient}
          PM(I6,M6) = AX0;
        
          AX0 = 0x0000;        {2nd a coefficient}
          PM(I6,M6) = AX0;
        
          AX0 = 0x0000;        {3rd a coefficient}
          PM(I6,M6) = AX0;
        
          AX0 = 0x0000;        {4th a coefficient}
          PM(I6,M6) = AX0;
        
          AX0 = 0x0000;        {5th a coefficient}
          PM(I6,M6) = AX0;
        
          AX0 = 0x0000;        {6th a coefficient}
          PM(I6,M6) = AX0;
        
          AX0 = 0x0000;        {7th a coefficient}
          PM(I6,M6) = AX0;
{
Since it is a circular buffer the I6 index register automatically points
to the start of a coefficient array after initialisation.
}
{
The readers are expected to fill in the values of a coefficient values
above accorinding to difference equation they want to implement.
}

        AX0 = 0x06;     {Number of Y coefficients = 6}                         
        L5 = AX0;       {Circular buffer with length 6}
        M5 = 1;         {buffer stride  = 1}
        I5 = ^Bcoeff;   {I5 points to the b coefficients}
        
        AX0 = 0x0000;        {1st b coefficient}
        PM(I5,M5) = AX0;
        
        AX0 = 0x0000;        {2nd b coefficient}
        PM(I5,M5) = AX0;
        
        AX0 = 0x0000;        {3rd b coefficient}
        PM(I5,M5) = AX0;
        
        AX0 = 0x0000;        {4th b coefficient}
        PM(I5,M5) = AX0;
        
        AX0 = 0x0000;        {5th b coefficient}
        PM(I5,M5) = AX0;
        
        AX0 = 0x0000;        {6th b coefficient}
        PM(I5,M5) = AX0;

{
Since it is a circular buffer the I5 index register automatically points
to the start of a coefficient array after initialisation.
}
{
The readers are expected to fill in the values of b coefficient values
above accorinding to difference equation they want to implement.
}

        M1 = 1;                               {Output array stride = 1}
        I1 = ^Yout;                           {I1 will point to output Array}
{
 After multiplication and addtion result is 32 bit hence we reserve double
 space for the output array with LSB of first output in 1st location.
}
        L1 = 0x0000C;                             
        CNTR = SR0;                           {lsb in first location}
{
We set all outputs to zero initially; We assume Causal system, no
outputs befor the instant t = 0.
}
        DO MakeOutputZero UNTIL CE;
MakeOutputZero:
        DM(I1,M1) = 0;
                                                        
        AX0 = 0x0000;
        DM(DAC_ADDR) = AX0;

        AX0 = 0x03E8;
        {This count will give a sampling frequency of 10 kHz}

        DM(tperiod)=AX0;        {load Tperiod and tcount with same count}
        DM(tcount)=AX0;         
        AX0 = 0x0000;
        DM(tscale)=AX0;         {set scaling register = 0 No scaling}


{
When you write 1 to a bit in ICNTL reg. you make that interrupt edge
sensetive. Similarly when you set a bit in the IMASK reg. you enable
that particular interrupt.
}
        ICNTL = b#10111;            {All interrupts edge sensitive}

        {Enable Timer intterupt and disable all  other interrupts}

        IMASK = b#100001;

        L0 = L6;
        M0 = 1;
        I0 = ^Xinput;
        CNTR = L6;
{
We set all outputs to zero initially; We assume Causal system, no
outputs befor the instant t = 0.
}

        DO MakeInputZero UNTIL CE;
MakeInputZero:
        DM(I0,M0) = 0;
        CNTR = 0x000A;

{
All the initialisations are done here and the ADC is given start of
conversion pulse.
}
      M3 = -2;
      M4 = -1;                     
      M2 = -1;                     
      MR = 0X0000;
      SR0 = 0;
      SR1 = 0;                                  {ay1 used for keeping sum}
      SE= -1;                       
	
      DM(Start_Of_Conv) = AX1;    {Give start of Convertion pulse to ADC}
      ENA TIMER;                  {enable the timer }
Wait_here:   JUMP Wait_here;        {Wait here forever}

{
The program comes out of this loop only when the timer count expires and
the timer causes and intterupt.
}
	NOP;
	NOP;

{*****************************************************************************}
{
Timer interrupt service routine. In this routine the processor reads data from
the ADC, masks the upper 4 bits performs the calulations for difference
eqution.
Outputs the final output to the DAC.
At the same time  it gives a start of conversion pulse to the ADC, this done
to get next dat on next timer interrupt.
}

Tmr_ISR:
        AY0=0x0000;
        AY1=0x0000;
        AX0=0x0000;
                                        
{
The 
}
        CNTR=L5;
{
Remember, I1 points to Outputs while I5 to b coefficients, for
calulations fetch data from these locations.
}
        MX0=DM(I1,M1),MY0=PM(I5,M5);    
                                        
        DO YCalculations UNTIL CE;

{
Calculations are, Sum = B coefficient * Previous Output.
}
                 MR=MX0*MY0 (SS);
{
Read MSB.
}
                 MX1=DM(I1,M1);        

{
Put lower byte of Sum in SR0 (LS byte of SR) and high byte in SR1. 
}
                 SR = LSHIFT MR0 (LO);
                 MR0 = MR1;        
{
High Byte.
}
                 SR = SR OR LSHIFT MR1 (HI);            
                 MR1 = MR2;
                 AR = SR0 + AY0;
                
                 AY0=AR;
{
Calulation of High byte of Data read in MX1.
}
                 MR= MR+ MX1*MY0 (US);
                 SR=LSHIFT MR0 (LO);
{
Read next b coefficients.
}
                 MY0=PM(I5,M5);

{
Adjust SR for result.
}
                 SR=SR OR LSHIFT MR1 (HI);
                 SR = SR AND LSHIFT MR2 BY 15 (LO);

{
Read one more previous output.
}
                 MX0=DM(I1,M1);
                 AR=SR0+AY1+C;
                                                        
                 AY1=AR;                                
                 AR=AX0+AF+C;                   
                 AF=PASS SR1;
{
Modify value in AR by adding to previos sum. MSB of Multiplication.
}
YCalculations:   AX0=AR;
{
Adjust AR for DAC.
} 
         AR=AX0+AF+C;
{
Modify Output and Input data pointers. Remember, output is 32 bits
while input is 16 bits.
}
         MODIFY(I1,M3);                 
         MODIFY(I1,M2);
         MR1=AY1;
         MODIFY(I5,M4);
         MR0=AY0;

         SR=SR OR LSHIFT MR1 BY -8 (LO);
         SR=SR OR ASHIFT AR BY 4 (HI);  
                                                
{
The ADC is Read here, next start of conversion pulse given to ADC, the
calulations are performed on the current and previous input samples.
The calulations for input are same as that for output only they
performed on 16 bit data.
}

	 MX0=0;
         MY0=0;
{
To mask upper 4 bits (12 bit ADC) of the value read from ADC.
}
         AY0=0x0FFF;                    
         AX0=DM(ADC_ADDR);             {read the ADC}
         DM(ADC_ADDR) = AX1;           {send a start of convertion pulse}
                                       {for next input}
         AR=AX0 AND AY0;               {adjust ADC reading}
         AY0=0x0800;
         AR=AR-AY0;                     
{
Write the input to input location.
}
         DM(I0,M0)=AR;                  
                                        
         MR=0;
         AY0=SR0;
{
Modify index I0 register.
}
        MODIFY(I0,M2);
        AY1=SR0;
{
Calculations are, Sum = Sum + a coefficient * Previous inputs.
Sum already contains multipliction and addition of previous outputs.
}
        CNTR=L6;                      {Length of loop is number of a coeff.}
        DO XCalculations UNTIL CE;                

        MR=MR+MX0*MY0 (US);
        MX0=DM(I0,M0);                {Read inputs}
XCalculations:
        MY0=PM(I6,M6);                {Read a coefficient}
                                        
        MR=MR+MX0*MY0 (US);            
                                        
{
Output in the SR register.
}
        SR=LSHIFT MR0 (LO);            
        SR=SR OR LSHIFT MR1 (HI);
        SR= SR OR LSHIFT MR2 BY 15 (HI);
        AR=SR0+AY0;                    

{
Write lower 16 bit data to output
}
        DM(I1,M1)=AR;
{
Write upper 16 bits of data.
}
        SR0=AR;
        AR=SR1+AY1+C;          
                                        
        DM(I1,M1)=AR;
{
Modify input and output pointers.
}
        MODIFY(I1,M3);                 
        MODIFY(I0,M2);

{
Output is in SR adjust it, get the upper 16 bits, mask upper 4 bits
and write the output to DAC.
}
         SR=LSHIFT SR0 BY -11 (LO);
         SR=SR OR ASHIFT AR BY -11 (HI);
         AR=PASS SR0;                   
         IF LT JUMP Negative;
         AY0=0X0800;                    
         AR=AR+AY0;                       
         JUMP WriteToDAC;                 
Negative:
         AR = -AR;
         AY0=0X07FF;                    
         AR=AR AND AY0;                 
         AR=AY0-AR;                     
WriteToDAC:
        DM(DAC_ADDR)=AR;               {write the output to DAC}

        RTI;                           {return from the timer interrupt}

.ENDMOD;
{End of the Diffeq module}
