/*
About this program:
This program is used to design Low Pass FIR filter with given cutoff
and sampling frequency.
The user has to input the sampling frequency, cutoff frequency, Number
of coefficients for the filter, and the window function to be applied
to the filter coefficients.
*/
#include<stdio.h>
#include<conio.h>
#include<graphics.h>
#include<math.h>
#include<stdlib.h>

main()
{
	int i,j,k,CoeffNum,CalNum,Count,choice;
	float freq,Sampfr,Nyquist,Pi,fn,Cutoff,Temp[13],Coeff[25],r1;
	float WinMultiplier;
	char ch;
	FILE *out;

        /*
        Get the value of Pi.
        */
        Pi = 22.0 / 7.0;

        /*Leave----------------------------------*/
        clrscr();
        for(j=0;j<80;j++)
                printf("");
        printf("\t  Low Pass FIR filter Design with rectangular window. \n");
        for(j=0;j<80;j++)
                printf("");
        /*Leave----------------------------------*/
        /*
        Get the cutoff frequency, sampling frequency.
        */
        printf("\n Input the value of cutoff frequency: ");
        scanf("%f",&freq);
        printf("\n Input the value of Sampling frequency: ");
        scanf("%f",&Sampfr);

        /*
        Verify nyqiust criterian.
        */
        Nyquist = Sampfr / freq;

        if(Nyquist < 2.0)
        {
                printf("\n**** ERROR ****"
                "\nAccording to the Nyquist Criterian"
                " Sampling frequency should be at least \n"
                "twice the waveform frequency.\n");
                for(j = 0;j < 80;j++)
                        printf("");
                printf("\nExiting...");
                exit(0);
        }

        /*
        Get the Number of coefficients for filter.
        The number of coefficient have to be odd to take care of the symmetry.
        */
	printf("\n Input the no. of filter coefficients ( < 26): ");
        scanf("%d",&CoeffNum);
        if(CoeffNum > 25 || CoeffNum % 2 == 0)
        {
                if(CoeffNum > 25)
                        printf("\n\nNumber of coefficients should be less than 25.");
                else
                        printf("\n\nNumber of coefficients should be ODD.");

                printf("\n\nExiting...");
                getch();
                exit(1);
        }

        /*
        Get the type of window to be applied to the coefficients.
        */
	printf("\n Input Your Choice:"
	"\n 1.Rectangular Window."
	"\n 2.Triangular Window."
	"\n 3.Hanning Window.\n"
	" 4.Hamming Window."
	"\n Choice: ");
        scanf("%d",&choice);

        /*
        Because of the symmetric shape of the SINC (sin x/x) we can calculate
        only half value  on the positive side and repeat the values on negative
        side.
        The function is,
                       Sin (Wcutoff * n)
                       -----------------
                            Pi * n
        n = -N/2 to N/2
        where
        Wcutoff = cutoff freq in rad.
        N = Number of coefficients - 1.
        */
	fn = Sampfr/2;
        Cutoff = (2 * Pi * freq) / Sampfr;
	r1 = freq/fn;
        CalNum = CoeffNum / 2;
        /*
        get value for n =0;
        */
        Temp[0] = r1;
        /*
        Get all the coefficients on positive side.
        */
	for(i=1;i<=CalNum;i++)
        {
                /*
                Calculate sinc function.
                */
                Temp[i]= sin(r1*i*22/7)/(i*22/7);
                switch(choice)
                {
                        case 1:
                                /*
                                Rectangular window.
                                */
                                WinMultiplier = 1;
                                break;
                        case 2:
                                /*
                                Triangular window.
                                */
				WinMultiplier = 1 - (float) i/(float) CalNum;
                                break;
                        case 3:
                                /*
                                Hamming window.
                                */
				WinMultiplier = 0.5*(1+cos(i*Pi/(float)CalNum));
				break;
                        case 4:
                                /*
                                Hanning window.
                                */
                                WinMultiplier = 0.54+0.46*cos(i*Pi/(float)CalNum);
                                break;
                        default:
                                WinMultiplier = 1;
                }
                /*
                Apply the window function.
                */
                Temp[i] = Temp[i] * WinMultiplier;
        }
        Count = 0;

        /*
        Get all the coefficients by reapeating on negative side.
        */
        for(i=0;i< CoeffNum;i++)
        {
		j = CalNum - i;
                if(j > 0)
		      Coeff[i] = Temp[CalNum - i];
                else
                {
                      Coeff[i] = Temp[Count];
                      Count++;
                }
        }
        /*
        Open the coefficient file coeff.dat and write the coefficients in it.
        */
        out = fopen("coeff.dat","w");
	fprintf(out,"%d",CoeffNum);
	fprintf(out,"\n0");
        for(i=0;i<CoeffNum;i++)
		fprintf(out,"\n%f",Coeff[i]);

        rewind(out);
        fclose(out);
}
