/*
About this program:
This program is used to design a notch filter with a notch at the given
frequency.
The notch filter is obtained by placing a zero on the unit circle at the
desired frequency.
The frequency is input by the user along with the sampling frequency.
The fiter coefficients are written to default coefficient file coeff.dat
*/
#include<stdio.h>
#include<conio.h>
#include<graphics.h>
#include<math.h>
#include<stdlib.h>

#define OriginX  320
#define OriginY  240
#define Radius   200
float ZeroY,ZeroX;

/*
To draw pole zeros in the Z plane.
*/
void DrawPolezero();
main()
{
	int i,j,k,CoeffNum,CalNum,Count;
	float freq,Sampfr,Nyquist,Pi,fn,Cutoff,Temp[13],Coeff[25],r1;
	char ch;
	FILE *out;
        /*
        Get the value of Pi.
        */
        Pi = 22.0 / 7.0;
        /*Leave---------------------------------------------------------*/
        clrscr();
        for(j=0;j<80;j++)
                printf("");
        printf("\t  Notch filter design using Zero-Pole-Zero method. \n");
        for(j=0;j<80;j++)
                printf("");
        /*Leave---------------------------------------------------------*/

        /*
        Get the notch frequency and sampling frequency.
        */
        printf("\n Input the value of cutoff frequency: ");
        scanf("%f",&freq);
        printf("\n Input the value of Sampling frequency: ");
        scanf("%f",&Sampfr);
        Nyquist = Sampfr / freq;

        /*
        Check the nyquist criterian.
        */
        if(Nyquist < 2.0)
        {
                printf("\n**** ERROR ****"
                "\nAccording to the Nyquist Criterian"
                " Sampling frequency should be at least \n"
                "twice the waveform frequency.\n");
                for(j = 0;j < 80;j++)
                        printf("");
                printf("\nExiting...");
                exit(0);
        }

	fn = Sampfr/2;

        Cutoff = (2 * Pi * freq) / Sampfr;
        CoeffNum = 3;

        /*
        Calculate the coefficients.
        */
        Coeff[0] = 1;
        Coeff[1] = - 2 * cos (Cutoff);
	Coeff[2] = 1;

        /*
        Write the coefficients to the file coeff.dat
        */
        out = fopen("coeff.dat","w");

	fprintf(out,"%d",CoeffNum);
	fprintf(out,"\n0");
        for(i=0;i<CoeffNum;i++)
		fprintf(out,"\n%f",Coeff[i]);

        rewind(out);
	fclose(out);
        /*
        Draw the poles and zeros on the Z plane.
        */
	ZeroX = cos(Cutoff);
	ZeroY = sin(Cutoff);
        DrawPolezero();
}

/*
This function is used to draw poles and zeros in the Z plane,
accoring to the real and imaginary value of the pole or zero.
For explaination of this function refer to coef2plt.c 
*/
void DrawPolezero()
{
 int gd,gm;
 float Tempval;
 int Xpos,Ypos;
         detectgraph(&gd,&gm);
         initgraph(&gd,&gm,"");
         graphresult();
         cleardevice();
         setbkcolor(0);
	 circle(OriginX,OriginY,Radius);
	 line(OriginX - 300,OriginY,OriginX + 300, OriginY);
	 line(OriginX,OriginY - 230, OriginX, OriginY + 230);
	 outtextxy(OriginX - 100,OriginY - 225,"Imaginary(I)");
	 outtextxy(OriginX + 220 ,OriginY + 5,"Real(X)");
	 outtextxy(OriginX + 15, OriginY + 10, "Z = 0");
	 outtextxy(OriginX + 15, OriginY - 10, "");
	 Tempval = 200.00 * ZeroX;
	 Xpos = (int) Tempval;
	 Tempval = 200.00 * ZeroY;
	 Ypos = (int) Tempval;
	 setcolor(3);
	 circle(OriginX + Xpos, OriginY + Ypos,4);
	 line(OriginX,OriginY,OriginX + Xpos, OriginY + Ypos);
	 outtextxy(OriginX + Xpos + 5, OriginY + Ypos + 10, "Z = Cos  - i Sin ");
	 circle(OriginX + Xpos, OriginY - Ypos,4);
	 line(OriginX,OriginY,OriginX + Xpos, OriginY - Ypos);
	 outtextxy(OriginX + Xpos + 5, OriginY - Ypos - 10, "Z = Cos  + i Sin ");
	 line(OriginX - 6,OriginY - 5,OriginX + 4,OriginY + 5);
	 line(OriginX + 4,OriginY - 5,OriginX - 6,OriginY + 5);
	 line(OriginX - 4,OriginY - 5,OriginX + 6,OriginY + 5);
	 line(OriginX + 6,OriginY - 5,OriginX - 4,OriginY + 5);
	 setcolor(15);
	 outtextxy(OriginX - 230, OriginY + 150,"Unit Circle");
	 settextstyle(0,0,2);
	 outtextxy(OriginX - 300, OriginY + 210,"Pole Zero Plot");
         getch();
         closegraph();
}
