/*
About this program:
This is a program for filter design using pole zero placement.
The program accepts the pole zero locations in terms of magitude r
in the Z plane and angle in radians.
The program then calculates the filter coefficients by first performing
all the cross products for the Z terms, and then coverting them to the
system transfer function in Z domain.
The coefficients are written to the default coefficient file coeff.dat
*/
#include<stdio.h>
#include<conio.h>
#include<stdlib.h>
#include<ctype.h>
#include<graphics.h>
#include<math.h>
#define OriginX  320
#define OriginY  240
#define Radius   200
#define Pole 1
#define Zero 0
float ZeroY,ZeroX;
float result[20],input[20][3];
int gd,gm;
int SqrEqMul(int);
void DrawPolezero(int);
main()
{
  int i,j,retval,n,Znum,Pnum,Xnum,Ynum;
  int flag;
  float Zmag[20],Zph[20],Pmag[20],Pph[20];
  float Xcoeff[20],Ycoeff[20],Zval[20],Pval[20],Zplt[20],Pplt[20];
  float TotMag;
  FILE *output;

  /*
  Set all inputs and outputs to zero.
  */
  for(i = 0;i < 20;i++)
  {
	result[i] = 0;
	input[2][i] = 0;
	input[1][i] = 0;
	input[0][i] = 0;
	Zmag[i] = 0;
	Zph[i] = 0;
	Pmag[i] = 0;
	Pph[i] = 0;
	Xcoeff[i] = 0;
	Ycoeff[i] = 0;
	Zval[i] = 0;
	Pval[i] = 0;
  }
  /*Leave----------------------------------------------------*/
  clrscr();
  for(j=0;j<80;j++)
        printf("");
  printf("\t\tFilter design by Pole and Zero placement.\n");
  for(j=0;j<80;j++)
        printf("");
  /*---------------------------------------------------------*/

  /*
  Get the number of zeros and poles, each pole and zero are added to the
  system along with their complex conjugate for the system to be real.
  */
  printf("\nEach Pole and Zero placed in the Z-plane will add it's "
	 "complex conjugate \nin the Z plane. This is done so that the "
	 "filter coefficients are real, \nand it is possible to implement"
	 " the filter.\n\n");
  printf("\nInput the Number of Zeros(< 5): ");
  scanf("%d",&Znum);
  printf("\nInput the Number of Poles(< 5): ");
  scanf("%d",&Pnum);

  if(Pnum != 0 && Pnum < Znum)
  {
	printf("\n\n\n****ERROR****\n"
	       "For an IIR filter the number of poles should not"
	       " be \nless than the number of zeros.");
	printf("\nExiting...");
	exit(0);
  }
  /*Leave----------------------------------------------------*/
  clrscr();
  for(j=0;j<80;j++)
        printf("");
  printf("\t\tFilter design by Pole and Zero placement.\n");
  for(j=0;j<80;j++)
	printf("");
  /*Leave----------------------------------------------------*/

  /*
  Get the position of zeros.
  */
  printf("\nThe position of a pole or zero is given as Z = r * EXP(jw)\n"
	 "where, r = radial distance.\n"
	 "and,   w = angle in radians.\n" );
  printf("\nInput the position of Zeros:\n");
  for(i = 0;i < Znum;i++)
  {
	printf("Zz(%d)-> r = ",i);
	scanf("%f",&Zmag[i]);
        Zplt[i] = Zmag[i];
	printf("Zz(%d)-> w = ",i);
	scanf("%f",&Zph[i]);
  }
  /*Leave------------------------------------------------*/
  clrscr();
  for(j=0;j<80;j++)
        printf("");
  printf("\t\tFilter design by Pole and Zero placement.\n");
  for(j=0;j<80;j++)
	printf("");
  /*Leave------------------------------------------------*/

  /*
  Get the position of zeros.
  */
  printf("\nThe position of a pole or zero is given as Z = r * EXP(jw)\n"
	 "where, r = radial distance.\n"
	 "and,   w = angle in radians.\n" );
  printf("\nInput the position of Poles:\n");
  for(i = 0;i < Pnum;i++)
  {
	printf("Zp(%d)-> r = ",i);
	scanf("%f",&Pmag[i]);
        Pplt[i] = Pmag[i];
	printf("Zp(%d)-> w = ",i);
	scanf("%f",&Pph[i]);
  }
  /*
  Since each zero is having acompplex conjgate, each term in the
  equation will be,
  (Z - a + jb)(Z - a -jb) = Z^2  - 2 * a + a^2 + b^2
  where a = r cos 
        a = r sin 
  */
  for(i = 0;i < Znum;i++)
  {
	if(Zmag[i] <= 0)
	{
		input[i][2] = 1;
		input[i][1] = 0;
		input[i][0] = 0;
		Zmag[i] = 1;
	}
	else
	{
                /*
                store the terms in array input.
                */
                input[i][2] = 1;
                input[i][1] = -1 * 2 * Zmag [i] * cos(Zph[i]);
                input[i][0] = Zmag[i] * Zmag[i];
	}        
  }
  n = Znum;
  /*
  Call the function SqrEqMul to get All the product terms for the zero's.
  */
  retval = SqrEqMul(n);
  /*
  Store the coefficients of the polynomial in Z. 
  */
  for(i = 0;i< 20; i++)
  {
    Zval[i] = result[i];
  }

  /*
  Repeat the same procedure for poles of the system.
  */
  for(i = 0;i < Pnum;i++)
  {
	if(Pmag[i] <= 0)
	{
		input[i][2] = 1;
		input[i][1] = 0;
		input[i][0] = 0;
		Pmag[i] = 1;
	}
	else
	{
		input[i][2] = 1;
                input[i][1] = -1 * 2 * Pmag[i] * cos(Pph[i]);
                input[i][0] = Pmag[i] * Pmag[i];
	}
  }
  n = Pnum;
  if(Pnum > 0)
	retval = SqrEqMul(n);
  for(i = 0;i< 20; i++)
  {
    Pval[i] = result[i];
  }
  /*
  The system function in Z domain is given by,
                a0 + a1 * Z^-1 + a2 * Z^-2 + a3 * Z^-3 + .... + aP * Z^-P
        H(Z) = --------------------------------------------------------------
                1 + b1 * Z^-1 + b2 * Z^-2 + b3 * Z^-3 + .... + bQ * Z^-Q
  This equation is obtained by dividing both the polynomials by the term having
  highest power in polynomial for the poles.
  */
  Xnum = Znum * 2;
  Ynum = Pnum * 2;

  /*
  Get the coefficients by performing the division.
  */
  TotMag = Pval[Ynum];
  for(i = Ynum ;i >= 0; i--)
  {
      Xcoeff[Ynum - i] = Zval[i] / TotMag;
      Ycoeff[Ynum - i] = Pval[i] / TotMag;
  }
  /*
  Write the coefficients to the file coeff.dat.
  */
  output = fopen("coeff.dat","w");
  if(output == NULL)
  {
	printf("\n Can't Open File Coeff.dat");
	exit(0);
  }
  fprintf(output,"%d",Ynum + 1);
  fprintf(output,"\n%d",Ynum);
  clrscr();

  printf("\nThe Filter coefficients are:"
	 "\nX coefficients: ");
  for(i = 0;i <= Ynum; i++)
  {
	fprintf(output,"\n%f",Xcoeff[i]);
        printf("\n%f",Xcoeff[i]);
  }
  printf("\nY coefficients: ");
  for(i = 1;i <= Ynum; i++)
  {
	fprintf(output,"\n%f",Ycoeff[i]);
	printf("\n%f",Ycoeff[i]);
  }
  fclose(output);

  getch();
/*---------------------------------------------------------------------*/
/*
The code after this is used to plot the poles and zeros  in the z plane.
It's presentation part of the program and readers needn't pay any attention
to it. For explaination reders can refer to the program coef2plt.c
*/

	 detectgraph(&gd,&gm);
	 initgraph(&gd,&gm,"");
	 graphresult();
         cleardevice();
         setbkcolor(0);
	 circle(OriginX,OriginY,Radius);
	 line(OriginX - 300,OriginY,OriginX + 300, OriginY);
	 line(OriginX,OriginY - 230, OriginX, OriginY + 230);
	 outtextxy(OriginX - 100,OriginY - 225,"Imaginary(I)");
	 outtextxy(OriginX + 220 ,OriginY + 5,"Real(X)");
	 outtextxy(OriginX + 15, OriginY + 10, "Z = 0");
         outtextxy(OriginX - 230, OriginY + 150,"Unit Circle");
	 settextstyle(0,0,2);
	 outtextxy(OriginX - 300, OriginY + 210,"Pole Zero Plot");
         for(i = 0;i < Znum; i++)
         {
                ZeroX = Zplt[i] * cos((double) Zph[i]);
		ZeroY = Zplt[i] * sin((double) Zph[i]);
		flag = Zero;
		DrawPolezero(flag);
         }
	 for(i = 0;i < Pnum; i++)
         {
                ZeroX = Pplt[i] * cos((double) Pph[i]);
		ZeroY = Pplt[i] * sin((double) Pph[i]);
		flag = Pole;
		DrawPolezero(flag);
         }
         setcolor(15);
         getch();
	 closegraph();
}
/*
The following function is used to calculate tthe polynomial from the given
polnomials of power 2.
*/
int SqrEqMul(int n)
{
  int i,j,k,Count,TempCount,ResCount,index,AddCount;
  float temp[20];
  /*
  Set all arrays to zero.
  */
  for(i = 0;i < 20;i++)
  {
	temp[i] = 0;
	result[i] = 0;
  }
  /*
  Result array contains coefficients of the polynomial.
  Initially polynoinal power is 2.
  */
  Count = 2;
  result[2] = input[0][2];
  result[1] = input[0][1];
  result[0] = input[0][0];

  /*
  Store result in temp array.
  */
  for(index = 0; index < 20; index++)
  {
	temp[index] = result[index];
	result[index] = 0;
  }

  for(i = 1;i< n ; i++)
  {

        TempCount = Count;
	Count = Count + 2;
	ResCount = Count;
        /*
        The following loop multiplies each term in the temp array by each
        next term in the input array and writes it in the result array.
        This increases the power of polynomial by two as the equtions
        contain square terms.
        */
	for(j = 2;j >= 0; j--)
	{
		AddCount = 0;
		for(k = TempCount;k >= 0;k--,AddCount++)
		{
			result[ResCount - AddCount] =  result[ResCount - AddCount] + temp[k] * input[i][j];
                } /* End of for*/
		ResCount--;
	}
        /*
        Store result in Temp array.
        */
	for(index = 0; index < 20; index++)
	{
	    temp[index] = result[index];
	    result[index] = 0;
	}
  }/*End of for*/

  /*
  Store the coefficients of the final polynomial in result array.
  */
  for(i = 0;i < 20;i++)
  {
	result[i] = temp[i];
  }
  return 0;
}

/*
This function is used to draw poles and zeros in the Z plane,
accoring to the real and imaginary value of the pole or zero.
For explaination of this function refer to coef2plt.c 
*/
void DrawPolezero(int flag)
{
 float Tempval;
 int Xpos,Ypos;
	 Tempval = 200.00 * ZeroX;
	 Xpos = (int) Tempval;
	 Tempval = 200.00 * ZeroY;
	 Ypos = (int) Tempval;
	 setcolor(3);
	 if(ZeroX == 0 || ZeroY == 0)
		setlinestyle(0,0,2);
	 else
		setlinestyle(0,0,0);
         if(flag == Zero)
         {

		circle(OriginX + Xpos, OriginY + Ypos,4);
                circle(OriginX + Xpos, OriginY - Ypos,4);
         }
         else
         {
                line(OriginX + Xpos - 4,OriginY + Ypos - 5,OriginX + Xpos + 4,OriginY + Ypos + 5);
                line(OriginX + Xpos - 4,OriginY + Ypos + 5,OriginX + Xpos + 4,OriginY + Ypos - 5);
                line(OriginX + Xpos - 4,OriginY - Ypos - 5,OriginX + Xpos + 4,OriginY - Ypos + 5);
                line(OriginX + Xpos - 4,OriginY - Ypos + 5,OriginX + Xpos + 4,OriginY - Ypos - 5);

         }
         line(OriginX,OriginY,OriginX + Xpos, OriginY - Ypos);
         line(OriginX,OriginY,OriginX + Xpos, OriginY + Ypos);
}
