/*
About this program:
This program is used to calculate magnitude and phase plots for the
given coefficients of a system.
The system function in Z domain is given as,

        a0 + a1 * Z^-1 + a2 * Z^-2 + a3 * Z^-3 + .... + aP * Z^-P
H(Z) = --------------------------------------------------------------
         1 + b1 * Z^-1 + b2 * Z^-2 + b3 * Z^-3 + .... + bQ * Z^-Q

The coefficients are read from the default coefficient file coeff.dat.
*/
#include<stdio.h>
#include<math.h>
#include<conio.h>
#include<graphics.h>
main()
{
	int grd,grm,i,j,k,x,y,n,y1,p,q;
        float t,r,h[501],max,l,m,Pcoeff[50],Qcoeff[50],step;
        float scale,reN,imN,reD,imD,ph[501],max1,Pi;
        float hN,hD,phN,phD,Tempval;
	char str[10],str1[10];
        FILE *f;

	clrscr();
        /*
        Get the value of Pi.
        */
        Pi = 22.0 /7.0;
        /*
        Open the file Coeff.dat and read the coefficients.
        */
	f=fopen("coeff.dat","r");
        p=0;
        q=0;
	fscanf(f,"%d",&p);
	if(p < 1)
	{
		printf("\nInvalid file format.");
		exit(0);
	}
        fscanf(f,"%d",&q);
        for(i = 0;i < p; i++)
	{
                if(feof(f))
                {
                        printf("\nInvalid file format.");
                        exit(0);
                }
                fscanf(f,"%f",&Pcoeff[i]);
	}
        if(q > 0)
        {
                for(i = 0;i < q; i++)
                {
                        if(feof(f))
                        {
                                printf("\nInvalid file format.");
                                exit(0);
                        }
                        fscanf(f,"%f",&Qcoeff[i]);
                }
        }
	rewind(f);
	fclose(f);

        /*
        Since we are plotting 500 points in the range 0 to Pi, (Here Pi
        corrosponds to Sampling freq / 2) We divide the range equally
        in seps of Pi/500.
        */
        step = Pi / 500.00;

        /*
        For all the values in the range 0 to Pi calulate Magnitude
        and phase for all coeffients in the numerator.
        Repeat the procedure for the denomenator polynomial.
        */
	for(r=0,i=0;i < 500; r=r+step,i++)
	{
		reN = 0;
		imN = 0;
                reD = 1.0;
		imD = 0;
                /*
                Magnitude is obtained as sum of all vector magnitudes of
                real and imaginary vector terms in Z domain.
                While phase is obtained as the sum of angles between them. 
                */
		for(j = 0; j < p; j++)
		{
                        /*
                        Numerator Terms.
                        */
                        reN = reN + Pcoeff[j] * cos(j * r);
                        imN = imN + Pcoeff[j] * sin(j * r);
		}
		if( q > 0)
		{
			for(j = 0; j < q; j++)
			{

                                /*
                                Denomenator Terms.
                                */
                                reD = reD - Qcoeff[j] * cos((j + 1) * r);
                                imD = imD + Qcoeff[j] * sin((j + 1) * r);
			}

		}
                phN = 0.0;
                phD = 0.0;
                hN = 0.0;
                hD = 1.0;
                /*
                Calculate the phase by taking Tan inverse of real and i
                maginary terms.
                */
		Tempval = imN / reN;
                if(Tempval < 0)
                        Tempval = -1 * Tempval;

                if(Tempval > 1.0e20)
                        phN = Pi/2;
                else
                        phN = atan(imN/reN);
                /*
                Calculate the magnitude of Numerator terms.
                */
                hN = sqrt(reN * reN + imN * imN);

                /*
                get the phase of denomeanator terms.
                */
                if(q > 0)
                {
                        Tempval = imD / reD;
                        if(Tempval < 0)
                                Tempval = -1 * Tempval;
                        if(Tempval > 1.0e20)
                                phD = Pi/2;
                        else
                                phD = atan(imD/reD);
                        hD = sqrt(reD * reD + imD * imD);
                }
                /*
                Store the magintude and phase terms in the magnitude and
                phase array.
                */
                ph[i] = phN - phD;
                h[i]= hN / hD;
        } /*End of for*/

        /*
        Find out the maximum phase angle and maximum magnitude.
        */
	max = h[0];
	max1 =ph[0];
	for(i=0;i<500;i++)
	{
		if(h[i]>max)
			max = h[i];
		if(ph[i]>max1)
			max1 = ph[i];
	}
        /*
        Normalise the values and convert them in to DBs, with maximum
        value corrosponding to 0 DBs.
        */
	for(i=0;i<500;i++)
	{
		h[i] = h[i]/max;
                h[i] = -1 * 20 * log10(h[i]);
	}
	max = h[0];
	for(i=0;i<500;i++)
		if(h[i]>max)
			max = h[i];
/*
The following code is used to polt the magnitude and phase response and the reader
need not go through it.
*/
	detectgraph(&grd,&grm);
	initgraph(&grd,&grm,"");
	graphresult();
	cleardevice();
	line(50,20,50,450);
	line(50,450,600,450);
	line(50,20,48,23);
	line(50,20,52,23);
	line(600,450,597,448);
	line(600,450,597,452);
	scale = 400/max;
	for(i=1;i<500;i++)
	{
		y1 = scale * h[i-1]+50;
		y = scale*h[i]+50;
		line(i+49,y1,i+50,y);
	}
	l=0;
	for(i=0;i<=10;i++)
	{
		sprintf(str,"%0.2f",l);
		line(i*50+50,448,i*50+50,452);
		outtextxy(i*50+48,454,str);
		l= l + step * 50;
	}
	l = max;
	m = max/10;
	for(i=0;i<=10;i++)
	{
                Tempval = l;
		if(Tempval < 0)
			Tempval *= -1;
		if(Tempval < 0.01)
			l = 0;
		gcvt(l,3,str1);
		line(48,450-i*40,52,450-i*40);
		outtextxy(10,450-i*40,str1);
		if(i<10)
			outtextxy(5,450-i*40,"-");
		l=l-m;
	}
	setlinestyle(2,2,1);
	y=scale*3+50;
	line(50,y,550,y);
	setlinestyle(0,0,2);
	outtextxy(0,20,"H(z)Db");
	outtextxy(5,y,"-3.00");
        outtextxy(530,470,"Fnormalized");
	outtextxy(530,20,"MAG. PLOT");
	getch();
	cleardevice();
	line(50,20,50,470);
	line(50,250,600,250);
	line(50,20,52,23);
	line(50,20,48,23);
	line(600,250,597,252);
	line(600,250,597,248);
	line(50,470,52,467);
	line(50,470,48,467);
	scale = 200/max1;
	for(i=1;i<500;i++)
	{
		y1=250-scale*ph[i-1];
		y=250-scale*ph[i];
		if(y>=479)
			y=479;
		if(y1>=479)
			y1=479;
		line(i+49,y1,i+50,y);
	}
	l=step * 50;
	for(i=1;i<=10;i++)
	{
		gcvt(l,3,str);
		line(i*50+50,248,i*50+50,252);
		outtextxy(i*50+48,254,str);
		l=l+ step * 50;
	}
	l=0;
	m=max1/10;
	for(i=0;i<=10;i++)
	{
		gcvt(l,3,str1);
		line(48,250-i*20,52,250-i*20);
		outtextxy(5,250-i*20,str1);
		l=l+m;
	}
	l=0;
	m=max1/10;
	for(i=0;i<=10;i++)
	{
		gcvt(l,3,str1);
		line(48,250+i*20,52,250+i*20);
		outtextxy(0,250+i*20,str1);
		l=l-m;
	}
	outtextxy(530,20,"PHASE PLOT");
        outtextxy(555,260,"fnormal");
	outtextxy(0,20,"ARG[H(Z)]rad");
	getch();
	closegraph();
}
